!! This module contains several routines for the calculation of the fluorescence yield and fluxes
!!
!!
!! C. Bacour 
!! 10/2015-03/2016
!! 
MODULE fluor

  USE constantes
  USE pft_parameters
  ! modules_used
  ! USE constantes_veg
  ! USE stomate_constants

  IMPLICIT NONE

  PUBLIC  :: mod_absleaf_from_chl, mod_absleaf_from_chl_sla, mod_absleaf740nm_from_sla, mod_kNPQt_driver, &
     mod_kNPQt_from_phiP0, mod_kNPQt_from_phiP6, mod_kNPQt_from_phiP9, mod_kNPQt_from_phiP9_3D,&
     mod_kNPQt_from_Lee, mod_kNPQt_from_vDT, &
     mod_SCOPE_parametric_driver, mod_SCOPE_parametric_740nm, mod_SCOPE_parametric_760nm,&
     mod_SCOPE_parametric_Cdm_740nm, mod_SCOPE_parametric_Cdm_760nm, &
     calc_ext_coef, calc_gross_func, calc_gross_ave_sza, &
     mod_frac_Fleaf740nm_to_APAR_vs_Cab, mod_kChlrel_from_chl


CONTAINS

  !! ========================================================================================================================
  !! This function calculates the leaf absorption as a function of the leaf chlorophyll content
  !! ========================================================================================================================
  FUNCTION mod_absleaf_from_chl(chlc, kjpindex) RESULT(absleaf)
    
    !! Declaration
    ! inputs
    INTEGER(i_std)                                :: kjpindex
    REAL(r_std), DIMENSION(kjpindex),  INTENT(in) :: chlc
    ! local
    REAL(r_std), DIMENSION(kjpindex)              :: absleaf
    REAL(r_std), SAVE, DIMENSION(4)               :: c_absleaf = (/0.90507126,0.61069852,12.131878,-0.086473994/)

    absleaf(:) = c_absleaf(1)*( 1 - c_absleaf(2)*EXP( -chlc(:) / c_absleaf(3) + c_absleaf(4) ))
    
  END FUNCTION mod_absleaf_from_chl


  !! ========================================================================================================================
  !! This function calculates the leaf absorption as a function of the leaf chlorophyll content + SLA
  !! ========================================================================================================================
  FUNCTION mod_absleaf_from_chl_sla(chlc, sla, kjpindex) RESULT(absleaf)
    
    !! Declaration
    ! inputs
    INTEGER(i_std)                                :: kjpindex
    REAL(r_std), DIMENSION(kjpindex),  INTENT(in) :: chlc
    REAL(r_std), INTENT(in)                       :: sla
    ! local
    REAL(r_std), DIMENSION(kjpindex)              :: absleaf
    REAL(r_std)                                   :: cdm
    REAL(r_std), SAVE, DIMENSION(7)               :: c = (/0.5786902, -0.14413878, 0.59760956, 0.7263141, 7.14562722, -8.31706638, 0.96043673/)

    cdm = 1.e-4 * 1/sla
    cdm = MAX(0.005, MIN(0.04,cdm)) !! bound wrt the range used for the calibration
    absleaf(:) = c(1)*cdm**2 + c(2)*cdm - c(3)*EXP(-chlc**c(4)/c(5) +c(6)*cdm ) +c(7)   

  END FUNCTION mod_absleaf_from_chl_sla


  !! ========================================================================================================================
  !! This function calculates the leaf absorption at 740nm as a function of SLA
  !! ========================================================================================================================
  FUNCTION mod_absleaf740nm_from_sla(sla) RESULT(absleaf)
    
    !! Declaration
    ! inputs
    REAL(r_std), INTENT(in)                       :: sla
    ! local
    REAL(r_std)                                   :: absleaf
    REAL(r_std)                                   :: cdm
    REAL(r_std), SAVE, DIMENSION(4)               :: c = (/0.6715,  0.1882, 14.5858,  1.1232/)

    cdm = 1.e-4 * 1/sla
    cdm = MAX(0.005, MIN(0.04,cdm)) !! bound wrt the range used for the calibration
    absleaf = c(1) - c(2)*EXP(-cdm*c(3) +c(4) )
    
  END FUNCTION mod_absleaf740nm_from_sla


  !! ========================================================================================================================
  !! This function calculates the fraction of leaf Fluorensce emitted at 740nm as a function of APAR
  !! APAR en umol.m-2.s-1
  !! ========================================================================================================================
  FUNCTION mod_frac_Fleaf740nm_to_APAR_vs_Cab(Cab,ps_string) RESULT(frac)
    
    !! Declaration
    ! inputs
    REAL(r_std)                                   :: Cab
    CHARACTER(LEN=3),  INTENT(in)                 :: ps_string !! 'PSI' or 'PSII'
    ! local
    REAL(r_std)                                   :: frac
    REAL(r_std), DIMENSION(2)                     :: coefs   
    REAL(r_std), SAVE, DIMENSION(2)               :: coefs_PSI  = (/0.91236507, 0.10085344/) !(/0.84031422, 0.01739803/)!(/0.8932268,0.00100199/)
    REAL(r_std), SAVE, DIMENSION(2)               :: coefs_PSII = (/0.74411911, 0.00369819/) !(/0.89137931, 0.00295657/)!(/9.59034270e-01,3.40854363e-04/)


    SELECT CASE (TRIM(ps_string))
    CASE('PS1')
        coefs = coefs_PSI
    CASE('PS2')
        coefs = coefs_PSII
    END SELECT

    frac = MAX(0.,1-coefs(1)*EXP(-coefs(2)*Cab))
    

    
  END FUNCTION mod_frac_Fleaf740nm_to_APAR_vs_Cab


  !! ========================================================================================================================
  !! This function calculates the relative specific absorption coefficient of chlorophylls (kChl/k) integrated over
  !! 400-750 nm based on Fluspect simulation by varying Cab (for a standard leaf)
  !! ========================================================================================================================
  FUNCTION mod_kChlrel_from_chl(chlc) RESULT(kChlrel)
    
    !! Declaration
    ! inputs
    !INTEGER(i_std)                                :: kjpindex
    !REAL(r_std), DIMENSION(kjpindex),  INTENT(in) :: chlc
    REAL(r_std), INTENT(in)                       :: chlc
    ! local
    !REAL(r_std), DIMENSION(kjpindex)              :: kChlrel
    REAL(r_std)                                   :: kChlrel
    REAL(r_std), SAVE, DIMENSION(4)               :: c = (/0.4680, 0.3839, 3.7832, 0.7744/)

    kChlrel =  1 - c(1)*EXP( -chlc**c(2) / c(3) + c(4) )
    
  END FUNCTION mod_kChlrel_from_chl


  !! ========================================================================================================================
  !! This function is the driver for the calculation of kNPQt
  !! ------------------------------------------------------------------------------------------------------------------------
  FUNCTION mod_kNPQt_driver(phiP, phiPo0, temp, PARin) RESULT(kNPQt)
  !FUNCTION mod_kNPQt_driver(phiP, phiPo0, temp) RESULT(kNPQt)
    
    !! Declaration
    ! inputs
    REAL(r_std),  INTENT(in)       :: phiP, phiPo0
    REAL(r_std),  INTENT(in)       :: temp
    REAL(r_std),  INTENT(in)       :: PARin

    ! local
    REAL(r_std)                    :: kNPQt

    SELECT CASE(fluor_model_kNPQ)
    CASE('FluOR0')
        kNPQt = mod_kNPQt_from_phiP0(phiP, phiPo0, temp)
    CASE('FluOR6')
        kNPQt = mod_kNPQt_from_phiP6(phiP, phiPo0, temp)
    CASE('FluOR9')
        kNPQt = mod_kNPQt_from_phiP9(phiP, phiPo0, temp)
    CASE('FluOR9_3D')
        kNPQt = mod_kNPQt_from_phiP9_3D(phiP, phiPo0, temp, PARin)
    CASE('Lee')
        kNPQt = mod_kNPQt_from_Lee(phiP, phiPo0)
    CASE('vanDerTol')
        kNPQt = mod_kNPQt_from_vDT(phiP, phiPo0)    
    ENDSELECT

  END FUNCTION MOD_KNPQT_DRIVER

  !! ----------------------------------------------------------------------
  !! This function calculates the relative rate constante of NPQ
  !! as a function 1-phiP/phiP0 and temperature
  !! 
  !! Parameter estimated for phiP0 = 0.806
  !! ----------------------------------------------------------------------
  
  !! -
  !! - Modele 0
  !! -
  FUNCTION mod_kNPQt_from_phiP0(phiP, phiPo0, temp) RESULT(kNPQt)   
    !! Declaration
    ! inputs
    REAL(r_std),  INTENT(in)       :: phiP, phiPo0
    REAL(r_std),  INTENT(in)       :: temp
    ! local
    REAL(r_std)                    :: kNPQt
    REAL(r_std)                    :: x
    REAL(r_std),SAVE               :: Tseuil = 0.
    REAL(r_std),SAVE, DIMENSION(4) :: coefs =  (/5.96612474,  1.93666393, -0.05165509,  0.84283963/) 
    x = 1-phiP/phiPo0  
    kNPQt = 0. * x
    IF (x .GT. 1.e-5) THEN
        IF (temp .LT. 273.15+Tseuil) THEN
            kNPQt = ( coefs(1)*x**coefs(2)) * EXP(coefs(3)*(Tseuil)+coefs(4))
        ELSE
            kNPQt = ( coefs(1)*x**coefs(2) ) * EXP(coefs(3)*(temp-273.15) + coefs(4))
        ENDIF
    ENDIF
  END FUNCTION mod_kNPQt_from_phiP0

  !! -
  !! - Modele 6
  !! -
  FUNCTION mod_kNPQt_from_phiP6(phiP, phiPo0, temp) RESULT(kNPQt)
    !! Declaration
    ! inputs
    REAL(r_std),  INTENT(in)       :: phiP, phiPo0
    REAL(r_std),  INTENT(in)       :: temp
    ! local
    REAL(r_std)                    :: kNPQt
    REAL(r_std)                    :: x
    REAL(r_std),SAVE               :: Tseuil = 0.5
    REAL(r_std),SAVE, DIMENSION(5) :: c = (/ 1.83805687,  1.996722,    0.59953595, -0.0939731 ,  2.03947803/)
    x = 1-phiP/phiPo0  
    kNPQt = 0. * x
    IF (x .GT. 1.e-5) THEN
        IF (temp .LT. 273.15+Tseuil) THEN
            kNPQt = (c(1)*x**c(2))*EXP(c(4)*(Tseuil)**c(3)+c(5))
        ELSE
            kNPQt = (c(1)*x**c(2))*EXP(c(4)*(temp-273.15)**c(3)+c(5))
        ENDIF
    ENDIF
  END FUNCTION mod_kNPQt_from_phiP6


  !! -
  !! - Modele 9
  !! -
  FUNCTION mod_kNPQt_from_phiP9(phiP, phiPo0, temp) RESULT(kNPQt)
    !! Declaration
    ! inputs
    REAL(r_std),  INTENT(in)       :: phiP, phiPo0
    REAL(r_std),  INTENT(in)       :: temp
    ! local
    REAL(r_std)                    :: kNPQt
    REAL(r_std)                    :: x
    REAL(r_std),SAVE               :: Tseuil = 2.5
    REAL(r_std),SAVE, DIMENSION(5) :: c = (/ 0.329493,    3.56446418,  2.20950091, -0.02012769,  3.62880121/)
    x = 1-phiP/phiPo0  
    kNPQt = 0. * x
    IF (x .GT. 1.e-5) THEN
        IF (temp .LT. 273.15+Tseuil) THEN
            kNPQt = c(1)*(1+c(2))*x**c(3)/(c(2)+x**c(3))*EXP(c(4)*(Tseuil)+c(5))
        ELSE
            kNPQt = c(1)*(1+c(2))*x**c(3)/(c(2)+x**c(3))*EXP(c(4)*(temp-273.15)+c(5))
        ENDIF
    ENDIF
  END FUNCTION mod_kNPQt_from_phiP9

  !! -
  !! - Modele 9 3D
  !! -
  FUNCTION mod_kNPQt_from_phiP9_3D(phiP, phiPo0, temp, PARin) RESULT(kNPQt)  
    !! Declaration
    ! inputs
    REAL(r_std),  INTENT(in)       :: phiP, phiPo0
    REAL(r_std),  INTENT(in)       :: temp
    REAL(r_std),  INTENT(in)       :: PARin
    ! local
    REAL(r_std)                    :: kNPQt
    REAL(r_std)                    :: x
    REAL(r_std),SAVE               :: Tseuil = -2.5
    REAL(r_std),SAVE, DIMENSION(6) :: c = (/1.60418604e+01,   5.73963883e+00,   2.16708270e+00,  -1.39755978e-02,  -4.36682170e-02,   5.76431816e-02/)
    x = 1-phiP/phiPo0  
    kNPQt = 0. * x
    IF (x .GT. 1.e-5) THEN
        IF (temp .LT. 273.15+Tseuil) THEN
            kNPQt = c(1)*(1+c(2))*x**c(3)/(c(2)+x**c(3))*EXP(c(4)*(Tseuil)+c(5))/PARin**c(6)
        ELSE
            kNPQt = c(1)*(1+c(2))*x**c(3)/(c(2)+x**c(3))*EXP(c(4)*(temp-273.15)+c(5))/PARin**c(6)
        ENDIF
    ENDIF   
  END FUNCTION mod_kNPQt_from_phiP9_3D




  !! ----------------------------------------------------------------------
  !! This function calculates the relative rate constante of NPQ
  !! using the model of Lee (2015)
  !! ----------------------------------------------------------------------
  FUNCTION mod_kNPQt_from_Lee(phiP, phiPo0) RESULT(kNPQt)
    
    !! Declaration
    ! inputs
    REAL(r_std),  INTENT(in)       :: phiP, phiPo0

    ! local
    REAL(r_std)                    :: kNPQt
    REAL(r_std)                    :: x

    x = 1-phiP/phiPo0
    kNPQt = (6.2473*x-0.5944)*x

  END FUNCTION mod_kNPQt_from_Lee

  !! ----------------------------------------------------------------------
  !! This function calculates the relative rate constante of NPQ
  !! using the model of Van Der Tol (2015)
  !! ----------------------------------------------------------------------
  FUNCTION mod_kNPQt_from_vDT(phiP, phiPo0) RESULT(kNPQt)
    
    !! Declaration
    ! inputs
    REAL(r_std),  INTENT(in)       :: phiP, phiPo0

    ! local
    REAL(r_std)                    :: kNPQt
    REAL(r_std)                    :: x, kN0, a, b

    kN0 = 2.48
    a = 2.83
    b = 0.114

    x = 1-phiP/phiPo0
    kNPQt = (1+b)*x**a / (b+x**a) * kN0

  END FUNCTION mod_kNPQt_from_vDT

  !! ======================================================================================================================== 



  !! ========================================================================================================================
  !! This function calculates the relative rate constante of reversible NPQ
  !! as a function of the quantum yield of photochemistry
  !! ========================================================================================================================
  FUNCTION mod_kNPQr(phiP) RESULT(kNPQr)
    
    !! Declaration
    ! inputs
    REAL(r_std),  INTENT(in)       :: phiP

    ! local
    REAL(r_std)                    :: kNPQr
    REAL(r_std),SAVE, DIMENSION(4) :: coefs = (/ 152037.92191972,  194947.44355378,  116975.95713418, 6496.19547318/)
    REAL(r_std), PARAMETER         :: max_kNPQr = 3.4366
    
    kNPQr = (coefs(1)-phiP*coefs(2))/(coefs(3)*phiP+coefs(4))
    kNPQr = MAX(0., kNPQr)
    kNPQr = MIN(kNPQr, max_kNPQr)
   
  END FUNCTION mod_kNPQr



  !! ========================================================================================================================
  !! This function calculates the sustainable rate constante of reversible NPQ
  !! as a function of temperature
  !! ========================================================================================================================
  FUNCTION mod_kNPQs( temp) RESULT(kNPQs)
    
    !! Declaration
    ! inputs
    REAL(r_std), INTENT(in)        :: temp !! Temperature in Celcius !!!
    ! local
    REAL(r_std)                    :: kNPQs
    REAL(r_std),SAVE, DIMENSION(5) :: coefs = (/ -142.63011283, -21.61745032, -19.09949444, 0.98455333, -1.00990414/)

    kNPQs  = coefs(1) / ( coefs(2) + coefs(3) * EXP( -coefs(4) * (temp-coefs(5) ) ) )

    
  END FUNCTION mod_kNPQs




  !! ========================================================================================================================
  !! Analytical version of the SCOPE (1.61) radiative transfer model used to 
  !! calculates the relative fluorescence of PSI and PSII depending on:
  !!  - LAI, Cab, ALA (mean leaf angle of the ellipsoidal distribution of Campbell, 1990), sun zenith angle, incident radiation
  !! The model is calibrated for 2 wavelengths: 740 and 760 nm
  !! ========================================================================================================================
  FUNCTION mod_SCOPE_parametric_driver(chlc, lai, ala, cossza, sla, radin, fluor_waveband, fluor_is_cdm_scope_parametric, ps_string )  RESULT(fluoPS)
    
    !! Declaration
    ! inputs
    REAL(r_std),  INTENT(in)       :: chlc !! leaf chlorophyll content (a+b) in ug/cm-2
    REAL(r_std),  INTENT(in)       :: lai  !! leaf area index
    REAL(r_std),  INTENT(in)       :: ala  !! mean leaf angle of the Campbell distribution function (degrees)
    REAL(r_std),  INTENT(in)       :: cossza  !! cosinus of the sun zenith angle  (unitless)
    REAL(r_std),  INTENT(in)       :: radin !! incoming shortwave radiation (W/m2)
    REAL(r_std),  INTENT(in)       :: sla  !! specific leaf area
    INTEGER(i_std),  INTENT(in)    :: fluor_waveband    !! wavelength (nm)
    LOGICAL,  INTENT(in)           :: fluor_is_cdm_scope_parametric
    CHARACTER(LEN=3),  INTENT(in)  :: ps_string    
    ! outputs
    REAL(r_std)                    :: fluoPS
    
    
    SELECT CASE (fluor_is_cdm_scope_parametric)
        CASE(.TRUE.)
            SELECT CASE (fluor_waveband)
            CASE(740)
                fluoPS = mod_SCOPE_parametric_Cdm_740nm(chlc, lai, ala, cossza, sla, radin, ps_string)       
            CASE(760)
                fluoPS = mod_SCOPE_parametric_Cdm_760nm(chlc, lai, ala, cossza, sla, radin, ps_string)
            ENDSELECT
            
        CASE(.FALSE.)
            SELECT CASE (fluor_waveband)
            CASE(740)
                fluoPS = mod_SCOPE_parametric_740nm(chlc, lai, ala, cossza, radin, ps_string)       
            CASE(760)
                fluoPS = mod_SCOPE_parametric_760nm(chlc, lai, ala, cossza, radin, ps_string)
            ENDSELECT

    ENDSELECT
           
    
  END FUNCTION mod_SCOPE_parametric_driver


  !! ----------------------------------------------------------------------
  !! Calculation of the relative fluorescence at 740 nm from a parametric version of SCOPE
  !! ----------------------------------------------------------------------
  FUNCTION mod_SCOPE_parametric_740nm(chlc, lai, ala, coszang, radin, ps_string) RESULT(fluoPS)
    ! inputs
    REAL(r_std),  INTENT(in)       :: chlc !! leaf chlorophyll content (a+b) in ug/cm-2
    REAL(r_std),  INTENT(in)       :: lai  !! leaf area index
    REAL(r_std),  INTENT(in)       :: ala  !! mean leaf angle of the Campbell distribution function (degrees)
    REAL(r_std),  INTENT(in)       :: coszang   !! cosinus of the  sun zenith angle  (unitless)
    REAL(r_std),  INTENT(in)       :: radin     !! incoming shortwave radiation (W/m2)
    CHARACTER(LEN=3),  INTENT(in)  :: ps_string !! 'PSI' or 'PSII'
    
    ! local
    REAL(r_std)                    :: fluoPS
    
    REAL(r_std),SAVE, DIMENSION(7) :: coefs_PSI = (/0.00332135,  0.73118528,  0.47726668,  1.01277694,  0.31580072,  0.88257661, -0.13622949 /)
    REAL(r_std),SAVE, DIMENSION(7) :: coefs_PSII = (/0.0063203,   0.73119409,  0.47724771,  1.01277817,  0.31581883,  0.88259676, -0.13623563 /)
    REAL(r_std), DIMENSION(7)      :: coefs         
    REAL(r_std)                    :: A,B,C
    REAL(r_std)                    :: cossza, cossza_min, cossza_max

    cossza_min = COS(60*pi/180.)
    cossza_max = COS(5*pi/180.)
    cossza = MAX(cossza_min, MIN(cossza_max, coszang) )

    SELECT CASE (TRIM(ps_string))
    CASE('PS1')
        coefs = coefs_PSI
    CASE('PS2')
        coefs = coefs_PSII
    END SELECT

    A = (1 + EXP(-coefs(2)*(chlc/10.))) 
    B = lai**coefs(3)
    C = ( 1 -  coefs(4)*EXP(-coefs(5)*(COS(ala*pi/180.)**coefs(6)/cossza**coefs(7))*lai ) )
    fluoPS = (coefs(1)*radin) /A / B * C
  


    ! WRITE(*,*) "DEBUG Radin", radin
    ! WRITE(*,*) "DEBUG ala", ala
    ! WRITE(*,*) "DEBUG cossza", cossza
    ! WRITE(*,*) "DEBUG lai", lai
    ! WRITE(*,*) "DEBUG A", A
    ! WRITE(*,*) "DEBUG B", B
    ! WRITE(*,*) "DEBUG C", C



  END FUNCTION  MOD_SCOPE_PARAMETRIC_740NM


  !! ----------------------------------------------------------------------
  !! Calculation of the relative fluorescence at 760 nm from a parametric version of SCOPE
  !! ----------------------------------------------------------------------
  FUNCTION mod_SCOPE_parametric_760nm(chlc, lai, ala, coszang,  radin,  ps_string) RESULT(fluoPS)
    ! inputs
    REAL(r_std),  INTENT(in)       :: chlc !! leaf chlorophyll content (a+b) in ug/cm-2
    REAL(r_std),  INTENT(in)       :: lai  !! leaf area index
    REAL(r_std),  INTENT(in)       :: ala  !! mean leaf angle of the Campbell distribution function (degrees)
    REAL(r_std),  INTENT(in)       :: coszang   !! cosinus of the sun zenith angle  (unitless)
    REAL(r_std),  INTENT(in)       :: radin     !! incoming shortwave radiation (W/m2)
    CHARACTER(LEN=3),  INTENT(in)  :: ps_string !! 'PSI' or 'PSII'
    
    ! local
    REAL(r_std)                    :: fluoPS
    
    REAL(r_std),SAVE, DIMENSION(7) :: coefs_PSI = (/0.0055897,   0.41778624,  0.72190613,  1.00370208,  0.24311237,  0.79708975, -0.12193872 /)
    REAL(r_std),SAVE, DIMENSION(7) :: coefs_PSII = (/0.00914009,  0.41775946,  0.72194169,  1.00370073,  0.24306202,  0.79704319, -0.1219408 /)
    REAL(r_std), DIMENSION(7)      :: coefs    
    REAL(r_std)                    :: A,B,C
    REAL(r_std)                    :: cossza, cossza_min, cossza_max

    cossza_min = COS(60*pi/180.)
    cossza_max = COS(5*pi/180.)
    cossza = MAX(cossza_min, MIN(cossza_max, coszang) )

    SELECT CASE (TRIM(ps_string))
    CASE('PS1')
        coefs = coefs_PSI
    CASE('PS2')
        coefs = coefs_PSII
    END SELECT
    
    A = (1 + EXP(-coefs(2)*(chlc/10.)))
    B = (1./(1+LAI**coefs(3)))
    C = EXP(-coefs(5)*(COS(ala*pi/180.)**coefs(6)/cossza**coefs(7) ) * lai )
    fluoPS = (coefs(1)*radin) / A 
    fluoPS = fluoPS * B * ( 1 -  coefs(4)*C )   
 
    
    ! WRITE(*,*) "DEBUG Radin", radin
    ! WRITE(*,*) "DEBUG ala", ala
    ! WRITE(*,*) "DEBUG cossza", cossza
    ! WRITE(*,*) "DEBUG lai", lai
    ! WRITE(*,*) "DEBUG A", A
    ! WRITE(*,*) "DEBUG B", B
    ! WRITE(*,*) "DEBUG C", C



  END FUNCTION  MOD_SCOPE_PARAMETRIC_760NM




  !! ----------------------------------------------------------------------
  !! Calculation of the relative fluorescence at 740 nm from a parametric version of SCOPE
  !! ----------------------------------------------------------------------
  FUNCTION mod_SCOPE_parametric_Cdm_740nm(chlc, lai, ala, coszang, sla, par, ps_string) RESULT(fluoPS)
    ! inputs
    REAL(r_std),  INTENT(in)       :: chlc !! leaf chlorophyll content (a+b) in ug/cm-2
    REAL(r_std),  INTENT(in)       :: lai  !! leaf area index
    REAL(r_std),  INTENT(in)       :: ala  !! mean leaf angle of the Campbell distribution function (degrees)
    REAL(r_std),  INTENT(in)       :: coszang   !! cosinus of the  sun zenith angle  (unitless)
    REAL(r_std),  INTENT(in)       :: sla  !! specific  leaf area 
    REAL(r_std),  INTENT(in)       :: par     !! incoming shortwave radiation (W/m2)
    CHARACTER(LEN=3),  INTENT(in)  :: ps_string !! 'PSI' or 'PSII'
    
    ! local
    REAL(r_std)                    :: fluoPS
    REAL(r_std)                    :: Cdm

    ! LAI [0.1; 5.5]    
    ! REAL(r_std),SAVE, DIMENSION(7) :: coefs_PSI = (/ 0.005755 , 0.693 , 0.820 , 22.917 , 0.187 , 0.807 , -0.141/)
    ! REAL(r_std),SAVE, DIMENSION(7) :: coefs_PSII = (/0.010951 , 0.693 , 0.820 , 22.917 , 0.187 , 0.807 , -0.141/)
    ! LAI [0.1; 7.5]
    REAL(r_std),SAVE, DIMENSION(7) :: coefs_PSI =  (/0.018174 , 0.071631 , 0.925415 , 24.561468 , 0.090972 , 0.692762 , -0.123321/)
    REAL(r_std),SAVE, DIMENSION(7) :: coefs_PSII = (/0.034582 , 0.071640 , 0.925398 , 24.560538 , 0.090982 , 0.692778 , -0.123318/)

    REAL(r_std), DIMENSION(7)      :: coefs         
    REAL(r_std)                    :: A,B,C
    REAL(r_std)                    :: cossza, cossza_min, cossza_max

    Cdm = 1.e-4 * 1/sla

    cossza_min = COS(60*pi/180.)
    cossza_max = COS(5*pi/180.)
    cossza = MAX(cossza_min, MIN(cossza_max, coszang) )

    ! PRINT *,' Cab =',chlc
    ! PRINT *,' LAI = ',lai
    ! PRINT *,' ALA = ', ALA*pi/180.
    ! PRINT *,' COSSZA =', coszang
    ! PRINT *,' CDM = ', Cdm
    ! PRINT *,' par = ', par
    SELECT CASE (TRIM(ps_string))
    CASE('PS1')
        coefs = coefs_PSI
    CASE('PS2')
        coefs = coefs_PSII
    END SELECT

    A = coefs(1)*par / (1. + EXP(-coefs(2)*(chlc/10.))) 
    B = 1./(1.+lai**coefs(3)) * 1./(1.+coefs(4)*Cdm)
    C = ( 1 -  EXP(-coefs(5)*(COS(ala*pi/180.)**coefs(6)/cossza**coefs(7))*lai ) )

    fluoPS = A*B*C
  
  END FUNCTION  MOD_SCOPE_PARAMETRIC_CDM_740NM


  !! ----------------------------------------------------------------------
  !! Calculation of the relative fluorescence at 760 nm from a parametric version of SCOPE
  !! ----------------------------------------------------------------------
  FUNCTION mod_SCOPE_parametric_Cdm_760nm(chlc, lai, ala, coszang, sla, par,  ps_string) RESULT(fluoPS)
    ! inputs
    REAL(r_std),  INTENT(in)       :: chlc !! leaf chlorophyll content (a+b) in ug/cm-2
    REAL(r_std),  INTENT(in)       :: lai  !! leaf area index
    REAL(r_std),  INTENT(in)       :: ala  !! mean leaf angle of the Campbell distribution function (degrees)
    REAL(r_std),  INTENT(in)       :: coszang   !! cosinus of the sun zenith angle  (unitless)
    REAL(r_std),  INTENT(in)       :: sla  !! specific  leaf area 
    REAL(r_std),  INTENT(in)       :: par     !! incoming shortwave radiation (W/m2)
    CHARACTER(LEN=3),  INTENT(in)  :: ps_string !! 'PSI' or 'PSII'
    
    ! local
    REAL(r_std)                    :: fluoPS
    REAL(r_std)                    :: Cdm
    
    ! LAI [0.1; 5.5]
    ! REAL(r_std),SAVE, DIMENSION(7) :: coefs_PSI = (/0.003339 , 0.407 , 0.694 , 29.084 , 0.249 , 0.816 , -0.130/)
    ! REAL(r_std),SAVE, DIMENSION(7) :: coefs_PSII = (/0.005459 , 0.407 , 0.694 , 29.081 , 0.249 , 0.816 , -0.130/)
    ! LAI [0.1; 7.5]
    REAL(r_std),SAVE, DIMENSION(7) :: coefs_PSI =  (/0.005614 , 0.433661 , 0.802436 , 32.473604 , 0.140397 , 0.688236 , -0.119660/)
    REAL(r_std),SAVE, DIMENSION(7) :: coefs_PSII = (/0.009175 , 0.433649 , 0.802384 , 32.471871 , 0.140426 , 0.688289 , -0.119673/)
    REAL(r_std), DIMENSION(7)      :: coefs    
    REAL(r_std)                    :: A,B,C
    REAL(r_std)                    :: cossza, cossza_min, cossza_max

    Cdm = 1.e-4 * 1/sla

    cossza_min = COS(60*pi/180.)
    cossza_max = COS(5*pi/180.)
    cossza = MAX(cossza_min, MIN(cossza_max, coszang) )

    SELECT CASE (TRIM(ps_string))
    CASE('PS1')
        coefs = coefs_PSI
    CASE('PS2')
        coefs = coefs_PSII
    END SELECT
    
    A = coefs(1)*par / (1. + EXP(-coefs(2)*(chlc/10.))) 
    B = 1./(1.+lai**coefs(3)) * 1./(1.+coefs(4)*Cdm)
    C = ( 1 -  EXP(-coefs(5)*(COS(ala*pi/180.)**coefs(6)/cossza**coefs(7))*lai ) )

    fluoPS = A*B*C


  END FUNCTION  MOD_SCOPE_PARAMETRIC_CDM_760NM


  !! ========================================================================================================================
  !! This function computes the extinction coefficient  according to 
  !! an ellipsoidal distibution of leaf angles  (Campbel, 1989; Wang & Jarvis, 1988) , as a FUNCTION of the mean 
  !! leaf inclination angle, the solar zenith angle 
  !! 
  !! use only 
  !! ========================================================================================================================
  FUNCTION calc_ext_coef(kjpindex,ala, cossza) RESULT(kext)

    !! Declaration
    ! inputs
    INTEGER(i_std), INTENT(in)                      :: kjpindex
    REAL(r_std), INTENT(in)                         :: ala       !! mean leaf angle of the Campbell distribution function (degrees)
    REAL(r_std), DIMENSION (kjpindex), INTENT(in)   :: cossza    !! cosinus of the sun zenith angle  (unitless)

    ! local
    REAL(r_std),DIMENSION (kjpindex)                      :: kext    
    REAL(r_std),DIMENSION (kjpindex)                       :: sza
    INTEGER(i_std)                                         :: j


    !!- Initialisation
    !!-----------------
    kext(:) = 0.5 !! standard version 

    !!-- Computation
    !!---------------
    sza(:) = MIN(ACOS(cossza(:)), 70.*pi/180.) !! bound sza to avoid to high values for kext
    
    kext(:) = calc_gross_func(kjpindex,ala,sza(:))/COS(sza(:))     
    
    !! DEBUG
    ! PRINT *,"Kext nadir:", calc_gross_func(kjpindex, 57.4, sza(:)*0.)/COS(sza(:)*0.)     
    ! PRINT *,"Kext nadir * clump", clumping(7)*calc_gross_func(kjpindex,57.4,sza(:)*0.)/COS(sza(:)*0.)     

    ! PRINT *,"Kext 70:", calc_gross_func(kjpindex, 57.4, sza(:)*70.*pi/180.)/COS(sza(:)*70.*pi/180.)     
    ! PRINT *,"Kext 70 * clump", clumping(7)*calc_gross_func(kjpindex,57.4,sza(:)*70.*pi/180.)/COS(sza(:)*70.*pi/180.)     

    ! STOP
    

  END FUNCTION calc_ext_coef


  ! FUNCTION calc_ext_coef1(ala, cossza) RESULT(kext)
  !   !! Declaration
  !   ! inputs
  !   REAL(r_std), INTENT(in)                         :: ala       !! mean leaf angle of the Campbell distribution function (degrees)
  !   REAL(r_std), INTENT(in)                         :: cossza    !! cosinus of the sun zenith angle  (unitless)
  !   ! local
  !   REAL(r_std)                      :: kext    
  !   REAL(r_std)                      :: sza
  !   kext = 0.5  
  !   sza = MIN(ACOS(cossza), 70.*pi/180.) !! bound sza to avoid to high values for kext   
  !   kext = calc_gross_func(1,ala,sza)/COS(sza)     
  ! END FUNCTION calc_ext_coef1



! FUNCTION calc_ext_coef(kjpindex,nvm, ala, cossza) RESULT(kext)

!     !! Declaration
!     ! inputs
!     INTEGER(i_std), INTENT(in)                      :: kjpindex,nvm
!     REAL(r_std), DIMENSION (nvm), INTENT(in)        :: ala       !! mean leaf angle of the Campbell distribution function (degrees)
!     REAL(r_std), DIMENSION (kjpindex), INTENT(in)   :: cossza    !! cosinus of the sun zenith angle  (unitless)

!     ! local
!     REAL(r_std),DIMENSION (kjpindex,nvm)                   :: kext    
!     REAL(r_std),DIMENSION (kjpindex)                       :: sza
!     INTEGER(i_std)                                         :: j


!     !!- Initialisation
!     !!-----------------
!     kext(:,:) = 0.5 !! standard version 

!     !!-- Computation
!     !!---------------
!     sza(:) = MIN(ACOS(cossza(:)), 70.*pi/180.) !! bound sza to avoid to high values for kext
    
!     DO j=2,nvm
!       kext(:,j) = calc_gross_func(kjpindex,ala(j),sza(:))/COS(sza(:))     
!     ENDDO

!     !! DEBUG
!     ! PRINT *,"Kext nadir:", calc_gross_func(kjpindex, 57.4, sza(:)*0.)/COS(sza(:)*0.)     
!     ! PRINT *,"Kext nadir * clump", clumping(7)*calc_gross_func(kjpindex,57.4,sza(:)*0.)/COS(sza(:)*0.)     

!     ! PRINT *,"Kext 70:", calc_gross_func(kjpindex, 57.4, sza(:)*70.*pi/180.)/COS(sza(:)*70.*pi/180.)     
!     ! PRINT *,"Kext 70 * clump", clumping(7)*calc_gross_func(kjpindex,57.4,sza(:)*70.*pi/180.)/COS(sza(:)*70.*pi/180.)     

!     ! STOP
    

!   END FUNCTION calc_ext_coef
  !! ========================================================================================================================


  !! ========================================================================================================================
  !! This function computes the averaged Gross function according for SZA
  !! between 0 and 76 degrees, which provides a value of about 0.8 for spherical distribution,
  !! consistent with the coefficient used for diffuse extinction coefficient of the Spitters 2 flux approach
  !! ========================================================================================================================
  FUNCTION calc_gross_ave_sza(ala) RESULT(GrossAve)

    !! Declaration
    ! inputs
    REAL(r_std), INTENT(in)                         :: ala

    ! local
    REAL(r_std)                                     :: GrossAve
    REAL(r_std), DIMENSION(77)                      :: sza
    INTEGER(i_std)                                  :: i
    
    GrossAve = 0.
    
    DO i = 1,77 
      sza(i) = (i-1)*pi/180.
    ENDDO
    GrossAve = SUM(calc_gross_func(77,ala,sza)/COS(sza))/77.  
    END FUNCTION calc_gross_ave_sza
  !! ========================================================================================================================


  !! ========================================================================================================================
  !! This function computes the Gross function according to 
  !! an ellipsoidal distibution of leaf angles  (Campbel, 1989; Wang & Jarvis, 1988) , as a FUNCTION of the mean 
  !! leaf inclination angle, the solar zenith angle 
  !! ========================================================================================================================
  FUNCTION calc_gross_func(kjpindex,ala,sza) RESULT(Grossf)

    !! Declaration
    ! inputs
    INTEGER(i_std), INTENT(in)                      :: kjpindex
    REAL(r_std),DIMENSION(kjpindex), INTENT(in)     :: sza
    REAL(r_std), INTENT(in)                         :: ala

    ! local
    REAL(r_std),DIMENSION(kjpindex)                 :: Grossf   !! parameters of the leaf angle distribution
    REAL(r_std)                                     :: ksi

    !!- Initialisation
    !!-----------------
    Grossf = 0.

    !!-- Computation
    !!---------------

      IF (ala .GE. 57.4) THEN   !! 57.4° <=> spherical distribution
          ksi = 151.515*(1-0.0107*ala)/ala
      ELSE
          ksi = 97.087*(1-0.0053*ala)/ala
      ENDIF
      Grossf(:) = (COS(sza(:))**2*(ksi**2-1)+1)**0.5 / (ksi+1.744*(ksi+1.182)**(-0.733))
    
    END FUNCTION calc_gross_func


  ! FUNCTION calc_gross_func1(ala,sza) RESULT(Grossf)
  !   !! Declaration
  !   ! inputs
  !   REAL(r_std),DIMENSION(kjpindex), INTENT(in)     :: sza
  !   REAL(r_std), INTENT(in)                         :: ala
  !   ! local
  !   REAL(r_std),DIMENSION(kjpindex)                 :: Grossf   !! parameters of the leaf angle distribution
  !   REAL(r_std)                                     :: ksi
  !   !!- Initialisation
  !   !!-----------------
  !   Grossf = 0.

  !   !!-- Computation
  !   !!---------------

  !     IF (ala .GE. 57.4) THEN   !! 57.4° <=> spherical distribution
  !         ksi = 151.515*(1-0.0107*ala)/ala
  !     ELSE
  !         ksi = 97.087*(1-0.0053*ala)/ala
  !     ENDIF
  !     Grossf = (COS(sza)**2*(ksi**2-1)+1)**0.5 / (ksi+1.744*(ksi+1.182)**(-0.733))
  !   END FUNCTION calc_gross_func1

  !! ========================================================================================================================






    
  !! ========================================================================================================================
  !! RTM2flux 740nm
  !! kextViewing : pour vza = 0
  !! rho et sigma calcules a partir de absLeaf a 740nm, tabule en fonction de SLA
  !! ========================================================================================================================
  FUNCTION RTM2flux(kdf, sla, kextViewing, nlai, laitab2,  Fleaf) RESULT(SIF)

    INTEGER(i_std), INTENT(in)                              :: nlai
    REAL(r_std), INTENT(in)                                 :: kdf,sla, kextViewing
    REAL(r_std), DIMENSION(nlai+1), INTENT(in)              :: laitab2                              
    !REAL(r_std), DIMENSION(nlai+1,2), INTENT(in)            :: Fleaf    !@FMbug
    REAL(r_std), DIMENSION(nlai,2), INTENT(in)              :: Fleaf
    !REAL(r_std), PARAMETER                                  :: vza = 0

    !! Local
    INTEGER(i_std)                                          :: jl
    REAL(r_std)                                             :: Fdir,Fdif, SIF !,SumDir
    REAL(r_std)                                             :: Idf, Idr, Idrdr, Idrdf
    REAL(r_std)                                             :: rho, sigma, absLeaf

    !! Calculate absorption at 740nm
    absLeaf = mod_absleaf740nm_from_sla(sla)

    !! rho et sigma
    sigma = 1-absLeaf
    rho = ((1-SQRT((1 - sigma)))/(1+SQRT((1 - sigma))))*(2/(1+1.6)) !! avec cos(vza=0) = 1
    
    SIF = 0.


    !! - Couche du bas - emission vers le haut
    !! Fleaf sont des flux hemispherique => la composante directionnelle correspond a Fleaf / (INT(COS(teta)domega) = PI)
    Fdir = 0.5*Fleaf(nlai,1)/pi
    Fdif = 0.5*Fleaf(nlai,2)/pi
    
    ! IF (Fleaf(1,1) .GT. 1) THEN
    !     PRINT *,'DEBUG',nlai,0,Fdif,Fdir
    ! ENDIF
    DO jl = nlai-1,2,-1 !! du bas vers le haut

      !! Contribution de la couche precedente
      Idf = Fdif * (1-rho) * EXP(-SQRT(1 - sigma)*kdf*laitab2(jl))
      Idr = Fdir * (1-rho) * EXP(-SQRT(1 - sigma)*kextViewing*laitab2(jl))
      Idrdr = Fdir * EXP(-kextViewing*laitab2(jl))
      Idrdf = Idr - Idrdr

      !! Ajout de l'emission propre
      Fdir = Idrdr + 0.5*Fleaf(jl,1)/pi
      Fdif = Idf + Idrdf + 0.5*Fleaf(jl,2)/pi
    
      !! Ajout de la reflectance sur la couche precedente du flux emis par la couche jl
      !   !! (rho est identique pour toutes les couches)
      IF (fluor_do_RefLayer_RTM2flux .EQ. .TRUE.) THEN
          Fdir = Fdir + (0.5*Fleaf(jl,1)/pi * rho) 
          Fdif = Fdif + (0.5*Fleaf(jl,2)/pi * rho) 
      ENDIF


      !! Check : somme contribution directe
      !SumDir = SumDir + Fleaf(jl,1)
      
      ! IF (Fleaf(1,1) .GT. 1) THEN
      !     PRINT *,'DEBUG',jl+1,jl,0.5*Fleaf(jl,1),Fdir,SumDir
      ! ENDIF
      
    ENDDO


    !! Ajout de l'emission de la couche TOC
    Fdir = Fdir + 0.5*Fleaf(1,1)/pi
    Fdif = Fdif + 0.5*Fleaf(1,2)/pi


    
    ! IF (Fleaf(1,1) .GT. 1) THEN
    !     PRINT *,'DEBUG',1,1,Fdif,Fdir
    !     !STOP
    ! ENDIF

    !! Sommation
    !! Flux hemispheriques => directionnels <=>  considerer qu'une seule portion du flux diffus dans visee nadir 
    ! => on divise par COS(teta)domega = cos(teta)sin(teta) dteta dphi = 2pi cos(teta)sin(teta) dteta (sur 0 - pi/2) = PI / Fait plus haut
    SIF = Fdir + Fdif
    
    
    END FUNCTION RTM2flux
  !! ========================================================================================================================    



END MODULE fluor

